# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2016 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de las vistas resumen financieras para una agente implementador. Este controlador es utilizado desde las secciones agentes.

class ResumenEmpleadosAgenteController < ApplicationController
  helper :resumen_proyecto

  before_filter :verificar_etapa

  def verificar_etapa
    if @agente.nil? || @agente.etapa.empty?
      msg_error _("Tiene que definir por lo menos una etapa para acceder a los informes")
      redirect_to :menu => :configuracion_agente, :controller => :datos_agente, :action => :etapas
    end
  end
  
  # en agente: se redirecciona por defecto a empleados
  def index
    redirect_to :action => :empleados
  end
  
  # OJO: El siguiente resumen de presupuesto de empleado, al igual que el resto de funcionalidades relacionadas con empleados no contemplan el tema de TASAS DE CAMBIO por que de momento se entiende que el presupuesto se hace en la moneda base de agente.
  def presupuesto_empleados
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}
    estado_presupuesto_empleados if params[:etapa]
    
    respond_to do |format|
      format.xls do
        @nombre = "Resumen Presupuesto de Personal por Proyectos " + @agente.nombre
        @resumen = @tablas.collect{|t| {tabla: t} }
        nom_fich = "resumen_presupuesto_personal_proyectos" + @agente.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
      end
      format.html
    end
  end
  
  def estado_presupuesto_empleados
    @etapa = Etapa.find_by_id(params[:etapa])
    @fecha_de_inicio = @etapa.fecha_inicio
    @fecha_de_fin = @etapa.fecha_fin
    @filas_empleados = []
    Empleado.where(agente_id: @agente.id).each do |e|
      @filas_empleados.push "id" => e.id, "nombre" => e.nombre
    end
    @titulo_fechas = [_("Fecha inicio") + ": " + @fecha_de_inicio.to_time.to_s, _("Fecha fin") + ": " + @fecha_de_fin.to_time.to_s]
    @tablas = [estado_presupuesto_agregado, estado_presupuesto_empleados_proyectos, estado_presupuesto_empleados_meses]
  end
  
  def estado_presupuesto_agregado
    datos = []
    columnas = [{"id" => 1, "nombre" => "Imputado al Agente"}, {"id" => 2, "nombre" => "Imputado a Proyectos"}]

    condiciones = { agente_id: @agente.id, fecha_inicio: @fecha_de_inicio.to_s, fecha_fin: @fecha_de_fin.to_s, moneda_id: @agente.moneda_principal.id }
    
    # Buscamos los presupuestos imputados al agente
    condiciones[:proyecto_imputado_id] = "isnull"
    VPresupuestoAgenteDetallado.empleados(condiciones).each do |vp|
      datos.push "columna_id" => 1, "fila_id" => vp.empleado_id, "importe" => vp.importe
    end
    # Buscamos los presupuestos imputados a proyectos
    condiciones[:proyecto_imputado_id] = "isnotnull"
    VPresupuestoAgenteDetallado.empleados(condiciones).each do |vp|
      datos.push "columna_id" => 2, "fila_id" => vp.empleado_id, "importe" => vp.importe
    end
    
    titulo_resumen = [_("Presupuesto de personal AGREGADO")] + @titulo_fechas 
    
    otros = { :fila_suma => true, :columna_suma => true, :fila_porcentaje => true, :ancho_fila => ((columnas.count + 2) * 105 + 300), :sin_truncar => true }
    tabla = { :titulo => titulo_resumen, :filas => @filas_empleados, :columnas => columnas, :datos => datos, :otros => otros }
    
    return tabla
  end
  
  def estado_presupuesto_empleados_proyectos
    datos = []
    columnas = [{"id" => nil, "nombre" => "Imputado al agente"}]
    
    @agente.proyectos_vinculados(@etapa).each do |p|
      columnas.push "id" => p.id, "nombre" => p.nombre
    end

    condiciones = { agente_id: @agente.id, fecha_inicio: @fecha_de_inicio.to_s, fecha_fin: @fecha_de_fin.to_s, moneda_id: @agente.moneda_principal.id}
    VPresupuestoAgenteDetallado.empleados_proyectos(condiciones).each do |vp|
      datos.push "columna_id" => vp.proyecto_imputado_id, "fila_id" => vp.empleado_id, "importe" => vp.importe
    end
        
    titulo_resumen = [_("Presupuesto de personal por PROYECTOS")] + @titulo_fechas 
    
    otros = { :fila_suma => true, :columna_suma => true,  :ancho_fila => ((columnas.count + 2) * 105 + 200) }
    tabla = { :titulo => titulo_resumen, :filas => @filas_empleados, :columnas => columnas, :datos => datos, :otros => otros }
    
    return tabla
  end
  
  def estado_presupuesto_empleados_meses
    datos = []
    columnas = []
    
    condiciones = {agente_id: @agente.id, moneda_id: @agente.moneda_principal.id}

    # Recorremos todos los periodos mensuales (incluye meses no enteros)
    for mes in (1..@etapa.periodos)
      condiciones[:fecha_inicio] = @fecha_de_inicio + (mes-1).month
      condiciones[:fecha_fin] = condiciones[:fecha_inicio] + 1.month  - 1.day
      condiciones[:columna_id] =  mes
      # Definimos la cabecera de la columna
      fecha_crono = I18n.l(condiciones[:fecha_inicio], :format => "%B %Y")
      columnas.push({"id" => mes, "nombre" => fecha_crono})
      VPresupuestoAgenteDetallado.empleados(condiciones).each do |vp|
        datos.push "columna_id" => mes, "fila_id" => vp.empleado_id, "importe" => vp.importe
      end
      
    end

        
    titulo_resumen = [_("Presupuesto de personal por MESES")] + @titulo_fechas 
    
    otros = { :fila_suma => true, :columna_suma => true, :ancho_fila => ((columnas.count + 2) * 105 + 200) }
    tabla = { :titulo => titulo_resumen, :filas => @filas_empleados, :columnas => columnas, :datos => datos, :otros => otros }
    
    return tabla
  end


  
end
